#version 120
#extension GL_ARB_shader_texture_lod : enable	
/*
!! DO NOT REMOVE !!
This code is from Chocapic13' shaders
Read the terms of modification and sharing before changing something below please !
!! DO NOT REMOVE !!
*/

//ADJUSTABLE VARIABLES//
	
	//#define Watercolor_Vanila								//Pure texture based water. Only enable one.
	//#define Watercolor_Clear								//Clear-ish water. Only enable one.
	//#define Watercolor_Tropical							//Weak green-ish water. Only enable one.
	//#define Watercolor_Legacy								//Strong blue water. Only enable one.
	//#define Watercolor_Classic							//Weak light blue water. Only enable one.
	#define Watercolor_Original								//Strong dark blue water. Only enable one.
	
	#define WaterParallax									//Enable Parallax for water.
		#define WaterParallax_Height 0.8					//Set wave height [0.2 0.4 0.6 0.8 1.0 1.2]
		#define WaterParallax_Samples 4						//Set looping value. Higher value gives less error [1 2 4 8 16 32 64]
	//#define WaterRipple									//Creates ripple effect near the player. Doesn't affect other players.
	
	//#define RPSupport
	
	//#define WorldTimeAnimation
	#define AnimationSpeed 1.0								//Set shader's speed. [0.1 0.25 0.5 0.75 1.0 2.0 3.0 4.0]

	#define NORMAL_MAP_MAX_ANGLE 1.0
	#define POM_MAP_RES 128.0										//Set to resourcepack size. You don't need to change it if you don't use resourcepack with shader support. [16.0 32.0 64.0 128.0 256.0 512.0]
	#define POM_DEPTH 0.10											//Set POM's depth. [0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50]

//ADJUSTABLE VARIABLES//	

/* Here, intervalMult might need to be tweaked per texture pack.
   The first two numbers determine how many samples are taken per fragment.  They should always be the equal to eachother.
   The third number divided by one of the first two numbers is inversely proportional to the range of the height-map. */

const vec3 intervalMult = vec3(1.0, 1.0, 1.0/POM_DEPTH)/POM_MAP_RES * 1.0;

const float MAX_OCCLUSION_DISTANCE = 22.0;
const float MIX_OCCLUSION_DISTANCE = 18.0;
const int   MAX_OCCLUSION_POINTS   = 50;
const float bump_distance = 64.0;				//Bump render distance: tiny = 32, short = 64, normal = 128, far = 256
const float pom_distance = 32.0;				//POM render distance: tiny = 32, short = 64, normal = 128, far = 256
const float fademult = 0.1;
const float PI = 3.1415927;

varying vec4 color;
varying vec2 texcoord;
varying vec2 lmcoord;
varying float mat;
varying float dist;

varying vec3 binormal;
varying vec3 normal;
varying vec3 tangent;
varying vec3 wpos;
varying vec3 viewVector;

uniform sampler2D texture;
uniform sampler2D normals;
uniform sampler2D noisetex;

#ifdef RPSupport
varying vec4 vtexcoordam; // .st for add, .pq for mul
varying vec4 vtexcoord;
uniform sampler2D specular;
#endif

uniform int worldTime;
uniform int isEyeInWater;
uniform float wetness;
uniform float rainStrength;
uniform float frameTimeCounter;
uniform vec3 cameraPosition;
uniform vec3 previousCameraPosition;

const float mincoord = 1.0/4096.0;
const float maxcoord = 1.0-mincoord;

#ifdef RPSupport
vec2 dcdx = dFdx(vtexcoord.st*vtexcoordam.pq);
vec2 dcdy = dFdy(vtexcoord.st*vtexcoordam.pq);

vec4 readTexture(in vec2 coord)
{
	return texture2DGradARB(texture,fract(coord)*vtexcoordam.pq+vtexcoordam.st,dcdx,dcdy);
}

vec4 readNormal(in vec2 coord)
{
	return texture2DGradARB(normals,fract(coord)*vtexcoordam.pq+vtexcoordam.st,dcdx,dcdy);
}
#endif

#ifdef WorldTimeAnimation
float frametime = worldTime/20.0*AnimationSpeed;
#else
float frametime = frameTimeCounter*AnimationSpeed;
#endif

float rainx = clamp(rainStrength, 0.0f, 1.0f)/1.0f;

vec2 dx = dFdx(texcoord.xy);
vec2 dy = dFdy(texcoord.xy);

float wave(float n) {
return sin(2 * PI * (n));
}

float getnoise(vec2 pos) {
	return abs(fract(sin(dot(pos ,vec2(18.9898f,28.633f))) * 4378.5453f));
}

float waterH(vec3 posxz,vec3 offset) {
float final = 0.0;
vec3 pos = posxz + offset;
//Noise
float noise = 0;
noise+= texture2D(noisetex,(pos.xz+vec2(frametime)*0.5-pos.y*0.2)/1024.0* 1.1).r*1.0;
noise+= texture2D(noisetex,(pos.xz-vec2(frametime)*0.5-pos.y*0.2)/1024.0* 1.5).r*0.8;
noise-= texture2D(noisetex,(pos.xz+vec2(frametime)*0.5+pos.y*0.2)/1024.0* 2.5).r*0.6;
noise+= texture2D(noisetex,(pos.xz-vec2(frametime)*0.5-pos.y*0.2)/1024.0* 5.0).r*0.4;
noise-= texture2D(noisetex,(pos.xz+vec2(frametime)*0.5+pos.y*0.2)/1024.0* 8.0).r*0.2;

//Ripple
float ripple = 0;
pos = posxz + offset*4.0;
#ifdef WaterRipple
vec3 camtrail = (cameraPosition.xyz-previousCameraPosition)*2;
float rpos = length((pos-cameraPosition.xyz)*vec3(1.0,2.5,1.0)-vec3(0.0,1.5,0.0));
float rpos0 = rpos;
float rmult = 0.0;
for(int i = 0; i < 24; i++){
	float temp = length((pos-cameraPosition.xyz+camtrail*i*0.25)*vec3(1.0,2.5,1.0)-vec3(0.0,1.5,0.0))+i/8;
	rpos0 = min(rpos0,temp);
	rmult = max(rmult,(2.0*(1-i*0.0625)-temp)*0.5);
	}

ripple = sin(rpos0*10-frametime*6)*rmult;
ripple = clamp(ripple*min(length(rpos),1.0),-1.0,1.0)*0.2;
#endif

final = (noise/pow(max(dist,1.0),0.4))*(1-abs(ripple)*2.0) + ripple;
return final;
}

vec3 getParallaxWaves(vec3 posxz, vec3 viewVector) {
	vec3 parallaxPos = posxz;
	float waveH = (waterH(posxz,vec3(0))*1.0-0.5)*WaterParallax_Height/WaterParallax_Samples;
	
	for(int i = 0; i < WaterParallax_Samples; i++){
		parallaxPos.xz += waveH*(viewVector.xy)/dist;
		waveH = (waterH(parallaxPos,vec3(0))*1.0-0.5)*WaterParallax_Height/WaterParallax_Samples;
	}
	return parallaxPos;
}

//////////////////////////////VOID MAIN//////////////////////////////
//////////////////////////////VOID MAIN//////////////////////////////
//////////////////////////////VOID MAIN//////////////////////////////
//////////////////////////////VOID MAIN//////////////////////////////
//////////////////////////////VOID MAIN//////////////////////////////

void main() {
	
	#ifdef Watercolor_Vanila
	vec3 watercolor = vec3(1.0);
	float wateropacity = 0.8;
	#endif
	
	#ifdef Watercolor_Clear
	vec3 watercolor = vec3(0.02,0.08,0.14);
	float wateropacity = 0.3;
	#endif
	
	#ifdef Watercolor_Tropical
	vec3 watercolor = vec3(0.1,0.52,0.6);
	float wateropacity = 0.4;
	#endif
	
	#ifdef Watercolor_Legacy
	vec3 watercolor = vec3(0.0,0.3,0.7);
	float wateropacity = 0.7;
	#endif
	
	#ifdef Watercolor_Classic
	vec3 watercolor = vec3(0.1,0.4,0.7);
	float wateropacity = 0.4;
	#endif
	
	#ifdef Watercolor_Original
	vec3 watercolor = vec3(0.03,0.07,0.12);
	float wateropacity = 0.8;
	#endif
	
	float iswater = float(mat > 0.04 && mat < 0.07);
	
	vec4 raw = texture2D(texture, texcoord.xy);
	
	#ifdef RPSupport
	vec2 adjustedTexCoord = vtexcoord.st*vtexcoordam.pq+vtexcoordam.st;
	if (dist < MAX_OCCLUSION_DISTANCE) {
			if ( viewVector.z < 0.0 && readNormal(vtexcoord.st).a < 0.99 && readNormal(vtexcoord.st).a > 0.01)
		{
			vec3 interval = viewVector.xyz * intervalMult;
			vec3 coord = vec3(vtexcoord.st, 1.0);
			for (int loopCount = 0;
					(loopCount < MAX_OCCLUSION_POINTS) && (readNormal(coord.st).a < coord.p);
					++loopCount) {
				coord = coord+interval;

			}
			if (coord.t < mincoord) {
				if (readTexture(vec2(coord.s,mincoord)).a == 0.0) {
					coord.t = mincoord;
					discard;
				}
			}
			adjustedTexCoord = mix(fract(coord.st)*vtexcoordam.pq+vtexcoordam.st , adjustedTexCoord , max(dist-MIX_OCCLUSION_DISTANCE,0.0)/(MAX_OCCLUSION_DISTANCE-MIX_OCCLUSION_DISTANCE));
		}

	}
		raw = texture2DGradARB(texture, adjustedTexCoord, dcdx, dcdy);
	#endif
	
	vec4 tex = vec4(vec3(raw.b + (raw.r+raw.g)),max(wateropacity*(1-isEyeInWater*0.5),0.2));
	tex *= vec4(watercolor,1);
	
	#ifdef Watercolor_Vanila
	tex.rgb = raw.rgb*color.rgb;
	tex.a = wateropacity;
	#endif
	
	if (iswater < 0.9) tex = raw*color;
	
	vec3 posxz = wpos.xyz;
	#ifdef WaterParallax
	posxz = getParallaxWaves(posxz,viewVector);
	#endif

	float deltaPos = 0.1;
	float h0 = waterH(posxz,vec3(0.0,0.0,0.0));
	float h1 = waterH(posxz,vec3(deltaPos,0.0,0.0));
	float h2 = waterH(posxz,vec3(-deltaPos,0.0,0.0));
	float h3 = waterH(posxz,vec3(0.0,0.0,deltaPos));
	float h4 = waterH(posxz,vec3(0.0,0.0,-deltaPos));
	
	float xDelta = ((h1-h0)+(h0-h2))/deltaPos;
	float yDelta = ((h3-h0)+(h0-h4))/deltaPos;
	
	vec3 newnormal = vec3(xDelta,yDelta,1.0-xDelta*xDelta-yDelta*yDelta);
	
	vec4 frag2 = vec4((normal) * 0.5f + 0.5f, 1.0f);
	
	if (iswater > 0.9) {
		vec3 bump = newnormal;
			bump = bump;
			
		
		float bumpmult = WaterParallax_Height*0.04;	
		
		bump = 	bump * vec3(bumpmult, bumpmult, bumpmult) + vec3(0.0f, 0.0f, 1.0f - bumpmult);
		mat3 tbnMatrix = mat3(tangent.x, binormal.x, normal.x,
							tangent.y, binormal.y, normal.y,
							tangent.z, binormal.z, normal.z);
		
		frag2 = vec4(normalize(bump * tbnMatrix) * 0.5 + 0.5, 1.0);
	}
	else {
	#ifdef RPSupport
	vec3 bump = texture2DGradARB(normals, adjustedTexCoord, dcdx, dcdy).rgb*2.0-1.0;
	float bumpmult = NORMAL_MAP_MAX_ANGLE*(1.0-wetness*lmcoord.t*0.9);
	bump = bump * vec3(bumpmult, bumpmult, bumpmult) + vec3(0.0f, 0.0f, 1.0f - bumpmult);
	mat3 tbnMatrix = mat3(  tangent.x, binormal.x, normal.x,
							tangent.y, binormal.y, normal.y,
							tangent.z, binormal.z, normal.z);

	frag2 = vec4(normalize(bump * tbnMatrix) * 0.5 + 0.5, 1.0);
	#endif
	}
	
	vec4 spec = vec4(0);
	#ifdef RPSupport
	spec = texture2DGradARB(specular, adjustedTexCoord.st, dcdx, dcdy);
	#endif
	
	float matb = mat;
	if (iswater < 0.9 && tex.a > 0.9) matb = 0.15;
	
/* DRAWBUFFERS:0246 */

	gl_FragData[0] = tex;
	gl_FragData[1] = frag2;	
	gl_FragData[2] = vec4(lmcoord.t, matb, lmcoord.s, 1.0);
	gl_FragData[3] = spec;
}